import pickle
from collections import defaultdict 
import numpy as np

# Load Data.
islam_original = pickle.load(open('data/pkl_format/islam90_original.pkl', 'rb'))
jones_original = pickle.load(open('data/pkl_format/jones_original.pkl', 'rb'))

islam_sword = pickle.load(open('data/pkl_format/islam90_sword.pkl', 'rb'))
jones_sword = pickle.load(open('data/pkl_format/jones_sword.pkl', 'rb'))

islam_dd = pickle.load(open('data/pkl_format/islam90_dd.pkl', 'rb'))
jones_dd = pickle.load(open('data/pkl_format/jones_dd.pkl', 'rb'))

islam_dp = pickle.load(open('data/pkl_format/islam90_dp.pkl', 'rb'))
jones_dp = pickle.load(open('data/pkl_format/jones_dp.pkl', 'rb'))

islam_pdp = pickle.load(open('data/pkl_format/islam90_pdp.pkl', 'rb'))
jones_pdp = pickle.load(open('data/pkl_format/jones_pdp.pkl', 'rb'))

islam_cnn = pickle.load(open('data/pkl_format/islam90_cnn.pkl', 'rb'))
jones_cnn = pickle.load(open('data/pkl_format/jones_cnn.pkl', 'rb'))

islam_scooby = pickle.load(open('data/pkl_format/islam90_scooby.pkl', 'rb'))
jones_scooby = pickle.load(open('data/pkl_format/jones_scooby.pkl', 'rb'))

islam_threadom = pickle.load(open('data/pkl_format/islam90_threadom.pkl', 'rb'))
jones_threadom = pickle.load(open('data/pkl_format/jones_threadom.pkl', 'rb'))

islam_fiefdom = pickle.load(open('data/pkl_format/islam90_fiefdom.pkl', 'rb'))
jones_fiefdom = pickle.load(open('data/pkl_format/jones_fiefdom.pkl', 'rb'))

islam_deepdom = pickle.load(open('data/pkl_format/islam90_deepdom.pkl', 'rb'))
jones_deepdom = pickle.load(open('data/pkl_format/jones_deepdom.pkl', 'rb'))


# Chain Lists.
islam_chains = list(islam_original.keys())
jones_chains = list(jones_original.keys())

# All Residue Numbers for both Islam90 and Jones Datasets.
num_dict = pickle.load(open('data/pkl_format/num_dict.pkl', 'rb'))

def getDomainIdx(in_dict):
    return set(in_dict.values())


def getCounts(dom_idx, pred_chain_dict, corr_chain_dict, num_chain_list):
    # For a given domain, construct a binary vector to show where the correct and predicted assignments are for each domain.
    pred_vec = []
    corr_vec = []
    for n in num_chain_list:
        # Build the prediction vector.
        if pred_chain_dict[n] == dom_idx:
            pred_vec.append(1)
        else:
            pred_vec.append(0)        
        # Build the correct vector.
        if corr_chain_dict[n] == dom_idx:
            corr_vec.append(1)
        else:
            corr_vec.append(0)
    # Count TP, FP, TN, FN.
    tp = 0.
    fp = 0.
    tn = 0.
    fn = 0.
    for i in range(len((pred_vec))):
        if (pred_vec[i] == 1) and (corr_vec[i] == 1):
            tp += 1            
        elif (pred_vec[i] == 1) and (corr_vec[i] == 0):
            fp += 1
        elif (pred_vec[i] == 0) and (corr_vec[i] == 1):
            fn += 1
        elif (pred_vec[i] == 0) and (corr_vec[i] == 0):
            tn += 1
    return tp, fp, tn ,fn


def calculateMetrics(corr_dict, pred_dict):
    # Running Counts.
    tot_tp = 0.
    tot_fp = 0.
    tot_tn = 0.
    tot_fn = 0.
    # Loop over all chains.
    for c in corr_dict:
        dom_list = getDomainIdx(pred_dict[c]) # Get index numbers for each domain.
        num_chain_list = sorted(list(num_dict[c])) # Get residue numbers for the full chain.
        # Loop over each domain and add TP, FP, TN, FN to the running count.
        for d in dom_list:
            tp, fp, tn, fn = getCounts(d, pred_dict[c], corr_dict[c], num_chain_list)
            tot_tp += tp
            tot_fp += fp
            tot_tn += tn
            tot_fn += fn
    # Calculate Spec, Sens, and MCC.
    sens = tot_tp / (tot_tp + tot_fn)
    spec = tot_tn / (tot_tn + tot_fp)
    mcc = (tot_tp * tot_tn - tot_fp * tot_fn)/np.sqrt((tot_tp + tot_fp)*(tot_tp + tot_fn)*(tot_tn + tot_fp)*(tot_tn + tot_fn)) 
    print(sens, spec, mcc)
    return sens, spec, mcc

print('CNN')
islam_cnn_sens, islam_cnn_spec, islam_cnn_mcc = calculateMetrics(islam_original, islam_cnn)
jones_cnn_sens, jones_cnn_spec, jones_cnn_mcc = calculateMetrics(jones_original, jones_cnn)

print('SWORD')
islam_sword_sens, islam_sword_spec, islam_sword_mcc = calculateMetrics(islam_original, islam_sword)
jones_sword_sens, jones_sword_spec, jones_sword_mcc = calculateMetrics(jones_original, jones_sword)

print('DD')
islam_dd_sens, islam_dd_spec, islam_dd_mcc = calculateMetrics(islam_original, islam_dd)
jones_dd_sens, jones_dd_spec, jones_dd_mcc = calculateMetrics(jones_original, jones_dd)

print('DP')
islam_dp_sens, islam_dp_spec, islam_dp_mcc = calculateMetrics(islam_original, islam_dp)
jones_dp_sens, jones_dp_spec, jones_dp_mcc = calculateMetrics(jones_original, jones_dp)

print('PDP')
islam_pdp_sens, islam_pdp_spec, islam_pdp_mcc = calculateMetrics(islam_original, islam_pdp)
jones_pdp_sens, jones_pdp_spec, jones_pdp_mcc = calculateMetrics(jones_original, jones_pdp)

print('Threadom')
islam_threadom_sens, islam_threadom_spec, islam_threadom_mcc = calculateMetrics(islam_original, islam_threadom)
jones_threadom_sens, jones_threadom_spec, jones_threadom_mcc = calculateMetrics(jones_original, jones_threadom)

print('FIEFDOM')
islam_fiefdom_sens, islam_fiefdom_spec, islam_fiefdom_mcc = calculateMetrics(islam_original, islam_fiefdom)
jones_fiefdom_sens, jones_fiefdom_spec, jones_fiefdom_mcc = calculateMetrics(jones_original, jones_fiefdom)

print('Scooby')
islam_scooby_sens, islam_scooby_spec, islam_scooby_mcc = calculateMetrics(islam_original, islam_scooby)
jones_scooby_sens, jones_scooby_spec, jones_scooby_mcc = calculateMetrics(jones_original, jones_scooby)

print('DeepDom')
islam_deepdom_sens, islam_deepdom_spec, islam_deepdom_mcc = calculateMetrics(islam_original, islam_deepdom)
jones_deepdom_sens, jones_deepdom_spec, jones_deepdom_mcc = calculateMetrics(jones_original, jones_deepdom)

